@@ -252,7 +252,7 @@ class Output {
     void hex(short mode, size_t byte_offset, const char *data, size_t size);
 
     // jump to the next hex dump location (option -o)
-    void next(size_t byte_offset)
+    inline void next(size_t byte_offset)
     {
       if (offset - offset % flag_hex_columns != byte_offset - byte_offset % flag_hex_columns)
         done();
@@ -264,8 +264,15 @@ class Output {
       return offset % flag_hex_columns != 0;
     }
 
+    // if hex line is incomplete: complete it with done()
+    inline void complete(size_t off)
+    {
+      if (offset > 0 && offset < off)
+        done();
+    }
+
     // done dumping hex
-    void done()
+    inline void done()
     {
       if (offset % flag_hex_columns != 0)
       {
@@ -293,7 +300,8 @@ class Output {
       dump(*this),
       lock_(NULL),
       slot_(0),
-      lineno_(0)
+      lineno_(0),
+      flush_(flag_line_buffered)
   {
     grow();
   }
@@ -457,12 +465,33 @@ class Output {
       chr(tmp[--n]);
   }
 
-  // output a new line, flush if --line-buffered
+  // output a byte in octal
+  void oct(int i)
+  {
+    uint8_t b = static_cast<uint8_t>(i);
+    chr('0' + (b >> 6));
+    chr('0' + ((b >> 3) & 7));
+    chr('0' + (b & 7));
+  }
+
+  // output a new line and flush if --line-buffered
   void nl()
   {
     chr('\n');
 
-    if (flag_line_buffered)
+    check_flush();
+  }
+
+  // enable line buffered mode to flush each line to output
+  void set_flush()
+  {
+    flush_ = true;
+  }
+
+  // flush if output is line buffered
+  void check_flush()
+  {
+    if (flush_)
       flush();
   }
 
@@ -570,7 +599,10 @@ class Output {
   void binary_file_matches(const char *pathname, const std::string& partname);
 
   // output formatted match with options --format, --format-open, --format-close
-  void format(const char *format, const char *pathname, const std::string& partname, size_t matches, reflex::AbstractMatcher *matcher, bool body, bool sep);
+  void format(const char *format, const char *pathname, const std::string& partname, size_t matches, reflex::AbstractMatcher *matcher, bool body, bool next);
+
+  // output formatted inverted match with options -v --format, --format-open, --format-close
+  void format_invert(const char *format, const char *pathname, const std::string& partname, size_t matches, size_t lineno, size_t offset, const char *ptr, size_t size, bool next);
 
   // output a quoted string with escapes for \ and "
   void quote(const char *data, size_t size);
@@ -628,6 +660,7 @@ class Output {
   Buffers                       buffers_; // buffers container
   Buffers::iterator             buf_;     // current buffer in the container
   char                         *cur_;     // current position in the current buffer
+  bool                          flush_;   // true if output is line buffered
 
 };
 
