@@ -46,6 +46,9 @@
 // max collective length of ANSI CSI escape sequences collected when skipping lead text with skip>0
 #define SCREEN_MAX_CODELEN 256
 
+// enable to interpret backspace (CTRL-H), not recommended because search results may not match what is shown
+// #define WITH_BACKSPACE
+
 // emit ANSI SGR CSI sequence with one numeric parameter
 void Screen::CSI(int code, int num)
 {
@@ -259,8 +262,8 @@ bool Screen::setup(const char *title)
 
 #endif
 
-  // enable alternative screen buffer, alternate scroll, enable cursor w/o blinking, soft reset, clear screen, reset colors
-  put("\033[?1049h\033[?1007h\033[?25h\033[?12l\033[!p\033[2J\033[m");
+  // enable alternative screen buffer, alternate scroll, enable cursor w/o blinking, cursor keys normal mode, clear screen, reset colors
+  put("\033[?1049h\033[?1007h\033[?25h\033[?12l\033[?1l\033[2J\033[m");
 
   // set title, when provided as argument
   if (title != NULL)
@@ -392,7 +395,7 @@ int Screen::wchar_width(uint32_t wc)
       (wc >= 0xff00 && wc <= 0xff60) || // Fullwidth Forms
       (wc >= 0xffe0 && wc <= 0xffe6) ||
       (double_width_Emoji &&
-       (wc >= 0x1f18e && wc <= 0x1f7ff)) || // Emoticons etc
+       (wc >= 0x1f18e && wc <= 0x1f9ff)) || // Emoticons etc (updated)
       (double_width_CJK &&
        ((wc >= 0x20000 && wc <= 0x2fffd) || // CJK
         (wc >= 0x30000 && wc <= 0x3fffd)))));
@@ -513,6 +516,13 @@ void Screen::put(int row, int col, const char *text, size_t size, int skip, int
             ++text;
             break;
 
+#ifdef WITH_BACKSPACE
+          case '\b':
+            ++num;
+            ++text;
+            break;
+#endif
+
           case '\t':
             num -= 1 + (~(cols - num) & 7);
             ++text;
@@ -554,7 +564,7 @@ void Screen::put(int row, int col, const char *text, size_t size, int skip, int
               num -= width;
               if (num < 0)
               {
-                // cut a double wide character in half?
+                // cut a double wide character that does not fit in half
                 if (width == 2 && num == -1)
                 {
                   put(' ');
@@ -587,6 +597,20 @@ void Screen::put(int row, int col, const char *text, size_t size, int skip, int
       {
         int tab;
 
+#ifdef WITH_BACKSPACE
+        case '\b':
+          if (ptr > text)
+          {
+            const char *p = ptr;
+            while (p > text && (*--p & 0xc0) == 0x80)
+              continue;
+            put(text, p - text);
+            ++len;
+          }
+          text = ++ptr;
+          break;
+#endif
+
         case '\t':
           put(text, ptr - text);
           tab = 1 + (~(cols - len) & 7);
@@ -693,6 +717,7 @@ void Screen::put(int row, int col, const char *text, size_t size, int skip, int
             }
             else if (wc <= 0x1f)
             {
+              // display CTRL character
               put(text, ptr - text);
               if (*ptr == '\0' && nulls > 0)
               {
@@ -710,6 +735,7 @@ void Screen::put(int row, int col, const char *text, size_t size, int skip, int
             }
             else if (wc == 0x7f)
             {
+              // display control character 0x7f
               put(text, ptr - text);
               invert();
               put("^?", 2);
